﻿/*
 * Rasterize parts of the active document and save as SVG.
 *
 * Steps this script does:
 *
 * 1) Rasterize all layers and groups the name of which ends "_r"
 * 2) Save the document as SVG with the same file name it currently
 * 3) Undo the rasterization
 *
 * To see it in the File/Scripts/ menu of Illustrator, place
 * the script file in folder
 *
 * .../Adobe Illustrator CS6/Presets/.../Scripts/
 *
 */

var rasterizeOptions = new RasterizeOptions();
rasterizeOptions.transparency = true;
rasterizeOptions.resolution = 72;

function process (item) {
    $.writeln("Processing '" + item.name + "'");
    if (/_r$/.exec(item.name)) {
        $.writeln("Rasterizing..");
        $.writeln(item.visibleBounds);
        app.activeDocument.rasterize(item, item.visibleBounds, rasterizeOptions);
    } else {
        if (item.layers) {
            for (var i = 0; i < item.layers.length; i++) {
                process(item.layers[i]);
            }    
        }
        if (item.groupItems) {
            for (var i = 0; i < item.groupItems.length; i++) {
                process(item.groupItems[i]);
            }
        }
    }
}

process(app.activeDocument);

var exportOptions = new ExportOptionsSVG();
exportOptions.embedRasterImages = true;
exportOptions.embedAllFonts = true;
app.activeDocument.exportFile(app.activeDocument.fullName, ExportType.SVG, exportOptions);
app.activeDocument.saveAs(app.activeDocument.fullName);

app.undo();
